import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  QueryList,
  ViewChild,
  ViewChildren,
  ViewEncapsulation,
} from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';

import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  delay,
  map,
  merge,
  Observable,
  of,
  startWith,
  Subject,
  switchMap,
  take,
  takeUntil,
  timer,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { Partner } from '../models/partner';
import { PartnerPagination } from '../models/partner-pagination';
import { PartnerService } from '../services/partner.service';
import { ActivatedRoute, Router } from '@angular/router';
import {
  RegistComposeComponent,
  RegistComposeResult,
} from '../compose/regist-compose.component';
import { CashComposeComponent } from '../compose/cash-compose.component';
import { CommissionComposeComponent } from '../compose/commission-compose.component';
import { CompComposeComponent } from '../compose/comp-compose.component';
import { CouponComposeComponent } from '../compose/coupon-compose.component';
import { MessageComposeComponent } from '../compose/message-compose.component';

import { MatDialog } from '@angular/material/dialog';
import { SiteService } from 'app/modules/polyglot/site/services/site.service';
import { Site } from 'app/modules/proto/models/site_pb';
import { MemberService } from 'app/modules/polyglot/member/services/member.service';
import { MemberModel, MemberState } from 'app/modules/proto/models/member_pb';
import {
  ListMembersRequest,
  ListMembersResponse,
  UpdateMemberForStateRequest,
  UpdateMemberRequest,
} from 'app/modules/proto/c2se/member_pb';
import { MatCheckbox, MatCheckboxChange } from '@angular/material/checkbox';
import {
  ListMemberLevelsResponse,
  UpdateMemberLevelRequest,
} from 'app/modules/proto/c2se/member_level_pb';
import { MemberLevel } from 'app/modules/proto/models/member_level_pb';
import { Pagination } from 'app/modules/proto/protobuf/pagination_pb';
import { VendorService } from 'app/modules/polyglot/api/services/vendor.service';
import { environment } from 'environments/environment';
import { FuseAlertType } from '@fuse/components/alert';

@Component({
  selector: 'partner-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .partner-grid {
        /*                     cb   관리  아이디 등급  예금주  */
        grid-template-columns: 40px 80px auto 80px 80px;

        @screen sm {
          /*                     cb   관리  아이디 등급  예금주  */
          grid-template-columns: 40px 80px auto 80px 80px;
        }

        @screen md {
          /*                     cb   관리  아이디 등급  예금주 정산  보유금   충전  환전금 수익금 */
          grid-template-columns: 40px 80px auto 80px 80px 80px 140px 140px 140px 140px;
        }

        @screen lg {
          /*                     cb   관리  아이디 등급  예금주 정산  보유금  충전   환전금 수익금  가입   버튼 */
          grid-template-columns: 40px 80px auto 80px 80px 80px 140px 140px 140px 140px 100px 60px;
        }
      }
      .mat-tooltip {
        white-space: pre-line;
      }
      .daebon-color {
        color: aqua;
      }
      .bubon-color {
        color: darkorange;
      }
      .chongpan-color {
        color: red;
      }
      .maejang-color {
        color: green;
      }
      .member-color {
        color: blueviolet;
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  @ViewChildren('chkUsers') chkUsers!: QueryList<MatCheckbox>;

  memberTotalCount: number = 0;
  listMember!: MemberModel[];
  listPartner$!: Observable<MemberModel[] | undefined>;
  listMemberLevels$!: Observable<MemberLevel[] | undefined>;

  __isLevelOpened = false;
  __isAllPage = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartner?: Partner;
  pagination?: PartnerPagination;

  __members: MemberModel[] = [];
  __checkedUsers: string[] = [];

  alertTitle = '알림';

  title: string = '';
  btnTitle: string = '';
  memberClassId = '';

  signoutBlockConfigForm!: FormGroup;
  fuseAlertConfirmConfigForm!: FormGroup;

  changeResultShowAlert: boolean = false;

  alertConfig: { type: FuseAlertType; message: string } = {
    type: 'success',
    message: '회원 정보가 수정 되었습니다.',
  };

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _activatedRoute: ActivatedRoute,
    private _partnerService: PartnerService,
    private __memberService: MemberService,
    private __vendorService: VendorService,

    private router: Router,
    private _matDialog: MatDialog
  ) {
    switch (this.router.url) {
      case '/member/partner/all':
        this.title = '전체목록';
        this.btnTitle = '본사등록';
        this.memberClassId = environment.constants.classeIds.mainOffice;
        this.__isAllPage = true;
        break;
      case '/member/partner/main-office':
        this.title = '본사목록';
        this.btnTitle = '대본등록';
        this.memberClassId = environment.constants.classeIds.branch;
        break;
      case '/member/partner/branch':
        this.title = '대본목록';
        this.btnTitle = '부본등록';
        this.memberClassId = environment.constants.classeIds.division;
        break;
      case '/member/partner/division':
        this.title = '부본목록';
        this.btnTitle = '총판등록';
        this.memberClassId = environment.constants.classeIds.office;
        break;
      case '/member/partner/office':
        this.title = '총판목록';
        this.btnTitle = '매장등록';
        this.memberClassId = environment.constants.classeIds.store;
        break;
      case '/member/partner/store':
        this.title = '매장목록';
        this.btnTitle = '회원등록';
        this.memberClassId = environment.constants.classeIds.user;
        break;
      default:
        console.log(this.router.url);
        break;
    }

    this.fuseAlertConfirmConfigForm = this._formBuilder.group({
      title: '알림',
      message: '비밀번호를 변경 하시겠습니까?',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: true,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._partnerService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: PartnerPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products

    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result = data['listPartner'];
      this.listPartner$ = of(listMemberResult.getMembersList());
      this.listMember = listMemberResult.getMembersList();
      this.memberTotalCount = listMemberResult.getTotalCount();

      let listMemberLevelsResult: ListMemberLevelsResponse.Result =
        data['listMemberLevels'];

      this.listMemberLevels$ = of(listMemberLevelsResult.getMemberLevelsList());

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });

    this.__vendorService
      .listVendors()
      .then((l) => {
        console.log('list', l.getVendorsList());
      })
      .catch((e) => {
        console.log('error', e);
      });

    // Set signout block config
    this.__signoutBlockConfirmConfig();
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.detectChanges();

      // If the partner changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;

            let req = new ListMembersRequest();
            let paging = new Pagination();
            paging.setPage(this._paginator.pageIndex);
            paging.setPageSize(this._paginator.pageSize);

            req.setPagination(paging);
            let search = new ListMembersRequest.Search();
            search.setMemberClassId(environment.constants.classeIds.mainOffice);
            req.setSearch(search);

            return of(this.__memberService.listMembers(req));
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string): void {
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }

  viewSlot(id: string): void {
    let url: string = 'game/slot';
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __onClickRegist(event: MouseEvent, partner?: MemberModel): void {
    const dialogRef = this._matDialog.open(RegistComposeComponent, {
      data: {
        title: this.btnTitle,
        parent: partner,
        classId: this.memberClassId,
      },
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result: RegistComposeResult) => {
        console.log('Compose dialog was closed: ', result);
      });
  }
  __onClickCash(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CashComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCommission(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CommissionComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickComp(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CompComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickCoupon(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(CouponComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }
  __onClickMessage(event: MouseEvent): void {
    const dialogRef = this._matDialog.open(MessageComposeComponent, {
      data: { price: '', memo: '' },
    });

    dialogRef.afterClosed().subscribe((result) => {
      console.log('Compose dialog was closed!');
    });
  }

  private showAlert(type: FuseAlertType, message: string): void {
    // Set the alert
    this.changeResultShowAlert = true;
    this.alertConfig = {
      type,
      message,
    };
    this._changeDetectorRef.markForCheck();
  }

  private async showConfirmAlert(
    title: string,
    message: string
  ): Promise<boolean> {
    this.fuseAlertConfirmConfigForm.get('title')?.setValue(title);
    this.fuseAlertConfirmConfigForm.get('message')?.setValue(message);

    return new Promise<boolean>((resolve, reject) => {
      const dialogRef = this._fuseConfirmationService.open(
        this.fuseAlertConfirmConfigForm.value
      );

      dialogRef.afterClosed().subscribe((result) => {
        if (result === 'confirmed') {
          resolve(true);
        } else {
          resolve(false);
        }
      });
    });
  }

  private closeChangeAlert(): void {
    setTimeout(() => {
      this.changeResultShowAlert = false;
      this._changeDetectorRef.markForCheck();
    }, 5000);
  }

  __onChangeState(type: number): void {
    let count = 0;
    const totalCount = this.chkUsers.length;

    this.showConfirmAlert(
      this.alertTitle,
      '선택된 회원 상태를 모두 수정하시겠습니까?'
    ).then((result) => {
      if (!!result) {
        this.__checkedUsers.forEach((memberId: string) => {
          const req = new UpdateMemberForStateRequest();
          const findMember = this.listMember.find(
            (m) => m.getId() === memberId
          );
          req.setId(findMember!.getId());
          req.setState(type as any);

          this.__memberService
            .updateMemberForState(req)
            .then((result) => {
              count++;
            })
            .catch((reson) => {
              this.showAlert('error', '회원 상태 변경이 실패하였습니다.');
            })
            .finally(() => {
              if (count === totalCount) {
                this.showAlert('success', '회원 상태가 수정 되었습니다.');
                this.closeChangeAlert();
              }
            });
        });
      }
    });
  }

  __getRateTooltop(partner: MemberModel): string {
    const tempRate = 0;
    const memberSettlement = partner.getMemberSettlementSetting();

    const resultTooltip = `
      요율확인 \n 
      카지노-바카라: ${memberSettlement?.getRateCasinoBacara()}% \n 
      카지노-룰렛: ${memberSettlement?.getRateCasinoRoulette()}% \n 
      카지노-드레곤타이거: ${memberSettlement?.getRateCasinoDragonTiger()}% \n 
      카지노-그외:${memberSettlement?.getRateCasino()}% \n 
      슬롯:${memberSettlement?.getRateSlot()}% \n 
      카지노루징: ${memberSettlement?.getRateCasinoLoosing()}% \n 
      슬롯루징: ${memberSettlement?.getRateSlotLoosing()}%
    `;
    return resultTooltip;
  }

  __signoutBlockConfirmConfig(): void {
    this.signoutBlockConfigForm = this._formBuilder.group({
      title: '알림',
      message: '로그아웃되었습니다.',
      icon: this._formBuilder.group({
        show: true,
        name: 'heroicons_outline:exclamation',
        color: 'warn',
      }),
      actions: this._formBuilder.group({
        confirm: this._formBuilder.group({
          show: true,
          label: '확인',
          color: 'warn',
        }),
        cancel: this._formBuilder.group({
          show: false,
          label: '취소',
        }),
      }),
      dismissible: true,
    });
  }
  __onClickSignoutBlock(event: MouseEvent): void {
    // Open the dialog and save the reference of it
    const dialogRef = this._fuseConfirmationService.open(
      this.signoutBlockConfigForm.value
    );
  }

  __onChangeChkUsersAll(event: MatCheckboxChange): void {
    let checked = event.checked;
    this.chkUsers.forEach((c, i) => {
      c.checked = checked;
    });
    this.__updateCheckedUsers();
  }

  __onChangeChkUsers(event: MatCheckboxChange): void {
    this.__updateCheckedUsers();
  }

  __updateCheckedUsers(): void {
    this.__checkedUsers = [];
    this.chkUsers.forEach((c, i) => {
      if (c.checked) {
        this.__checkedUsers.push(c.value);
      }
    });
  }

  __onClickChangeLevel(member_level_id: string): void {
    console.log('member_level_id', member_level_id);
    let count = 0;
    const totalCount = this.chkUsers.length;

    this.showConfirmAlert(
      this.alertTitle,
      '선택된 회원 레벨을 모두 수정하시겠습니까?'
    ).then((result) => {
      if (!!result) {
        this.__checkedUsers.forEach((memberId: string) => {
          const req = new UpdateMemberRequest();
          const findMember = this.listMember.find(
            (m) => m.getId() === memberId
          );
          req.setId(findMember!.getId());
          req.setMemberLevelId(member_level_id);

          this.__memberService
            .updateMember(req)
            .then((result) => {
              count++;
            })
            .catch((reson) => {
              this.showAlert('error', '회원 레벨 수정이 실패하였습니다.');
            })
            .finally(() => {
              if (count === totalCount) {
                this.showAlert('success', '회원 레벨이 수정 되었습니다.');
                this.closeChangeAlert();
              }
            });
        });
      }
    });
  }

  __getMemberSettlementType(member: MemberModel): string | undefined {
    const type = member.getMemberSettlementSetting()?.getSettlementType();

    return type === 1 ? '롤링' : '';
  }

  __getChildMemberCount2(member: MemberModel): string[] {
    const childs = member.getChildrenCountList().join().toString();
    const childArr = childs.split(',');
    return childArr;
  }
  __getChildMemberCount(member: MemberModel): string {
    const childs = member.getChildrenCountList().join().toString();
    const childArr = childs.split(',');
    const tempColor = [
      'daebon-color',
      'bubon-color',
      'chongpan-color',
      'maejang-color',
      'member-color ',
    ];
    let result = '';
    const start = tempColor.length - childArr.length;

    for (let idx = 0; idx < childArr.length; idx++) {
      result += `<span class="${tempColor[idx + start]}" >${
        childArr[idx]
      }</span> `;
    }

    return result;
  }

  __getPartnerName(childArr: string[], idx: number): string {
    const partnerType = 5 - childArr.length + idx;
    let result = '';

    switch (partnerType) {
      case 0:
        result = '대본';
        break;
      case 1:
        result = '부본';
        break;
      case 2:
        result = '총판';
        break;
      case 3:
        result = '매장';
        break;
      case 4:
        result = '회원';
        break;
      default:
        break;
    }
    return result;
  }

  __getPartnerColor(childArr: string[], idx: number): string {
    const envPartnerColor = environment.constants.partnerColor;
    const colorType = 5 - childArr.length + idx;

    let result = '';
    switch (colorType) {
      case 0:
        result = envPartnerColor.branch;
        break;
      case 1:
        result = envPartnerColor.division;
        break;
      case 2:
        result = envPartnerColor.office;
        break;
      case 3:
        result = envPartnerColor.store;
        break;
      case 4:
        result = envPartnerColor.user;
        break;
      default:
        break;
    }
    return result;
  }

  __getMemberState(member: MemberModel): string {
    const state = member.getState();
    let resultStr = '';

    switch (state) {
      case MemberState.NONE:
        resultStr = '없음';
        break;
      case MemberState.NORMAL:
        resultStr = '정상';
        break;
      case MemberState.PENDING:
        resultStr = '대기';
        break;
      case MemberState.WITHDRAWAL:
        resultStr = '탈퇴';
        break;
      case MemberState.DORMANCY:
        resultStr = '휴면';
        break;
      case MemberState.BLACKLIST:
        resultStr = '블랙리스트';
        break;
      case MemberState.SUSPENDED:
        resultStr = '정지';
        break;
      default:
        break;
    }

    return resultStr;
  }
}
